@@ -274,7 +274,7 @@ class Connection(object):
         :param size: number of points to retrieve per series
         """
 
-        r = self._shooju_api.search_series(query, date_start, date_finish, size, fields=fields, scroll=True)
+        r = self._shooju_api.search_series(query, date_start, date_finish, size, fields=fields, scroll=True, query_size=500)
         scroll_id = r['scroll_id']
 
         while True:
@@ -881,13 +881,18 @@ class RemoteJob(object):
     That object is used to submit data to API
     """
 
-    def __init__(self, conn, job_id, batch_size):
+    def __init__(self, conn, job_id, batch_size, pre_hooks=None, post_hooks=None):
         """
         Initialized with connection and job_id.
 
+        Pre submit hooks and post commit hooks can be added to the job to perform actions before or after
+        data is submitted to shooju. The function should accept kwargs
+
         :param shooju.Connection conn: API connection
         :param job_id: job id
         :param int batch_size: size of cache before uploading series to the API
+        :param list pre_hooks: list of functions to be run before the job submits to shooju
+        :param list post_hooks: list of functions to be run after the job submits to shooju
         """
         self._conn = conn
         self._job_id = job_id
@@ -899,6 +904,9 @@ class RemoteJob(object):
         #{'fields':{},'points':{}} that !
         self._values = {}
 
+        self._pre_hooks = pre_hooks or []
+        self._post_hooks = post_hooks or []
+
     @property
     def job_id(self):
         return self._job_id
@@ -977,6 +985,7 @@ class RemoteJob(object):
         Submits the current cached data to server
         """
         #the submit part should traverse all of the sids and submit them
+        self._run_pre_submit_hooks()
         bulk_data = []
         for series, values in self._values.iteritems():
             #here we will construct a BULK API call
@@ -984,15 +993,45 @@ class RemoteJob(object):
             bulk_data.append(tmp_dict)
 
         #we send the bulk data at once
+        response = {}
         if bulk_data:
             try:
-                self._conn._post_bulk(bulk_data, self._job_id)
-            finally:
-                # flush cache to avoid re-raising the same exceptions
+                response = self._conn._post_bulk(bulk_data, self._job_id)
+            except ShoojuApiError:
+                # flush cache to avoid re-raising the same exceptions, but only for ShoojuApiErrors
                 self._values = {}
                 self._cur_batch_size = 0
+                raise
+        self._run_post_submit_hooks(response)
         return True
 
+    def add_post_submit_hook(self, fn):
+        """
+        Adds a hook that will be run after the cache is submitted to shooju
+        :param fn: function to be executed, it should accept kwargs
+        """
+        self._add_hooks(self._post_hooks, fn)
+
+    def add_pre_submit_hook(self, fn):
+        """
+        Adds a hook that will be run before the cache is submitted to shooju
+        :param fn: function to be executed, it should accept kwargs
+        """
+        self._add_hooks(self._pre_hooks, fn)
+
+    def _add_hooks(self, hook_list, fn):
+        if not hasattr(fn, '__call__'):
+            raise ValueError('hooks must be a function')
+        hook_list.append(fn)
+
+    def _run_pre_submit_hooks(self):
+        for fn in self._pre_hooks:
+            fn(job_id=self.job_id)
+
+    def _run_post_submit_hooks(self, response):
+        for fn in self._post_hooks:
+            fn(job_id=self.job_id, response=response)
+
 
 class GetBulk(object):
     """
